<?php 

defined( 'ABSPATH' ) || exit;

/* 
  Adding more options for wp_schedule_event parameter $recurrence
  The values supported by default are ‘hourly’, ‘twicedaily’, ‘daily’, and ‘weekly’
*/
add_filter( 'cron_schedules', 'taager_add_custom_recurrences' );
function taager_add_custom_recurrences( $schedules ) {
	$schedules['every_five_minutes'] = array(
		'interval' => 300,
		'display'  => __( 'Every 5 Minutes', 'taager-plugin' ),
	);
	$schedules['every_ten_minutes'] = array(
		'interval' => 600,
		'display'  => __( 'Every 10 Minutes', 'taager-plugin' ),
	);
	$schedules['every_half_an_hour']    = array(
		'interval' => 1800,
		'display'  => __( 'Every Half an Hour', 'taager-plugin' ),
	);
	$schedules['every_one_hour']    = array(
		'interval' => 3600,
		'display'  => __( 'Every One Hour', 'taager-plugin' ),
	);
	return $schedules;
}

add_action( 'ta_alternating_update_hook', 'taager_hourly_action_scheduler' );
function  taager_hourly_action_scheduler() {
	$wasProductsLastUpdated = get_option('ta_products_last_updated');
	if($wasProductsLastUpdated) {
		hourly_action_scheduler_taager_order_check();
		update_option('ta_products_last_updated', 0);
	} else {
		hourly_action_scheduler_taager_product_check();
		update_option('ta_products_last_updated', 1);
	}
}

/**
 * scheduled function that syncs taager products order hourly.
 *
 * @return void
 */
function hourly_action_scheduler_taager_order_check() {
	$args        = [
		'post_type'      => 'shop_order',
		'post_status'    => 'any',
		'posts_per_page' => -1,
		'meta_query'     => [
			'relation' => 'AND',
			[
				'key'     => '_ta_order_num',
				'compare' => 'EXISTS',
			],
		],
	];
	$order_lists = new WP_Query( $args );
	if ( $order_lists->have_posts() ) :
		while ( $order_lists->have_posts() ) :
			$order_lists->the_post();
			$wp_order_id     = get_the_ID();
			$ta_order_id = get_post_meta( $wp_order_id, 'taager_order_id', true );
			$ta_order_no  = get_post_meta( $wp_order_id, '_ta_order_num', true );
			if ($ta_order_id) {
				$query_params = array(
					'order_id' => $ta_order_id,
				);
			} else {
				$query_params = array(
					'order_num' => $ta_order_no,
				);
			}
			$response     = taager_call_API( 'GET', TAAGER_URL . '/orders', $query_params );
			
			$ta_status                  = ( isset( $response->data->status ) ) ? $response->data->status : '';
			$ta_suspendedReason         = ( isset( $response->data->suspendedReason ) ) ? $response->data->suspendedReason : '';
			$ta_customerRejectedReason  = ( isset( $response->data->customerRejectedReason ) ) ? $response->data->customerRejectedReason : '';
			$ta_deliverySuspendedReason = ( isset( $response->data->deliverySuspendedReason ) ) ? $response->data->deliverySuspendedReason : '';
			
			if(isset( $response->data->orderID )) {
				update_post_meta( $wp_order_id, 'taager_order_id', $response->data->orderID );
			}
			update_post_meta( $wp_order_id, 'taager_order_status', $ta_status );
			update_post_meta( $wp_order_id, 'taager_suspended_reason', $ta_suspendedReason );
			update_post_meta( $wp_order_id, 'taager_customer_rejected_reason', $ta_customerRejectedReason );
			update_post_meta( $wp_order_id, 'taager_delivery_suspended_reason', $ta_deliverySuspendedReason );
		endwhile;
	endif;
	wp_reset_postdata();
}

/**
 * scheduled function that syncs imported taager products hourly
 *
 * @return void
 */
function hourly_action_scheduler_taager_product_check() {
	global $wpdb;
	$args            = [
		'post_type'      => array('product', 'product_variation'),
		'post_status'    => 'publish',
		'posts_per_page' => -1,
		'meta_query'     => [
			'relation' => 'AND',
			[
				'key'     => 'taager_product',
				'compare' => '=',
				'value'   => '1',
			],
		],
	];
	$taager_products = new WP_Query( $args );
	if ( $taager_products->have_posts() ) :
		while ( $taager_products->have_posts() ) :
			$taager_products->the_post();
			$p_id = get_the_ID();

			$ta_shipping = get_post_meta( $p_id, 'ta__shipping_charge', true );
			$ta_id       = get_post_meta( $p_id, '_sku', true );

			$pid_param = '&prod_ids=' . urlencode( $ta_id );

			$response = taager_call_API( 'GET', TAAGER_URL . '/product/?' . $pid_param );

			if ( isset( $response->data ) ) {
				$all_ta_product     = $response->data;
				
				foreach($all_ta_product as $ta_product) {
					
					if ( $ta_product->isProductAvailableToSell ) {
						$stock_status = 'instock';
					} else {
						$stock_status = 'outofstock';
					}
					update_post_meta( $p_id, '_stock_status', $stock_status );
					$current_product_price = get_post_meta( $p_id, '_price', true );
					if ( 'yes' == $ta_shipping ) {
						$ta_provinces_name   = $wpdb->prefix . 'taager_provinces';
						$ta__select_max      = $wpdb->get_row( "SELECT * FROM $ta_provinces_name WHERE active_status=1 ORDER BY shipping_revenue DESC LIMIT 1" );
						$taager_max_shipping = $ta__select_max->shipping_revenue;
						$taager_profit       = get_post_meta( $p_id, '_ta_product_profit', true );
						$taager_price        = get_post_meta( $p_id, '_ta_product_price', true );

						$ta_new_min_price = $taager_price - $taager_profit + $taager_max_shipping;

						if ( intval( $current_product_price ) < intval( $ta_new_min_price ) ) {
							update_post_meta( $p_id, '_price', $ta_new_min_price );
						}
					} else {
						if ( intval( $current_product_price ) < intval( $product->productPrice ) ) {
							update_post_meta( $p_id, '_regular_price', $product->productPrice );
							update_post_meta( $p_id, '_price', $product->productPrice );
							update_post_meta( $p_id, '_ta_product_profit', $product->productProfit );
							$update_product_wc_status = array(
								'ID' => $p_id,
								'post_title'  => $product->productName,
								'post_status' => 'draft',
							);
							wp_update_post( $update_product_wc_status );
						}
					}
					update_post_meta( $p_id, '_ta_product_price', $ta_product->productPrice );
				}
			}
		endwhile;
		wp_reset_postdata();
	endif;
}

function taager_schedule_cron_events() {
	$current_cron_jobs = get_option('cron');
	$is_alternating_update_scheduled = false;

	if(is_array($current_cron_jobs) && count($current_cron_jobs) > 0) {
		$taager_get_cron_job_names = function ($carry, $item) {
			if(is_array($item)){
				foreach(array_keys($item) as $name) {
						$carry[] = $name;
					}
			}
			return $carry;
		};
		$cron_job_names = array_reduce($current_cron_jobs, $taager_get_cron_job_names, array() );
		foreach($cron_job_names as $name) {
			if($name === 'ta_hourly_update_hook') {
				wp_clear_scheduled_hook( 'ta_hourly_update_hook');
			}
			if($name === 'ta_alternating_update_hook') {
				$is_alternating_update_scheduled = true;
			}
		}
	}

	if(!$is_alternating_update_scheduled) {
		wp_schedule_event( time(), 'every_half_an_hour', 'ta_alternating_update_hook' );
	}
}

function taager_clear_cron_events() {
	$last_category_filter = get_option( 'ta_last_category_filter' );
	$last_name_filter = get_option( 'ta_last_name_filter' );
	$args = array( $last_category_filter, $last_name_filter );

	wp_clear_scheduled_hook( 'taager_import_products', $args );
	wp_clear_scheduled_hook( 'ta_hourly_update_hook' );
	wp_clear_scheduled_hook( 'ta_alternating_update_hook' );
}
